@@ -12,6 +12,7 @@ import threading
 import boto
 from boto.dynamodb.item import Item
 from boto.exception import DynamoDBResponseError
+import json
 
 
 log = logging.getLogger(__name__)
@@ -53,6 +54,36 @@ class autoincrement_int(int):
     pass
 
 
+class json_list(list):
+    """JSON-serialized list data type for use in your schemas.
+
+    From your object's point of view, it's a list with arbitrary nesting and
+    whatever you want to store in it.
+
+    In DynamoDB itself, though, it's stored as a JSON-serialized string.
+    """
+    def __init__(self, source_str=""):
+        if source_str:
+            super(json_list, self).__init__(json.loads(source_str))
+        else:
+            super(json_list, self).__init__()
+
+
+class json_dict(dict):
+    """JSON-serialized dict data type for use in your schemas.
+
+    From your object's point of view, it's a dict with arbitrary nesting and
+    whatever you want to store in it.
+
+    In DynamoDB itself, though, it's stored as a JSON-serialized string.
+    """
+    def __init__(self, source_str=""):
+        if source_str:
+            super(json_dict, self).__init__(json.loads(source_str))
+        else:
+            super(json_dict, self).__init__()
+
+
 class ConnectionBorg(object):
     """Borg that handles access to DynamoDB.
 
@@ -179,7 +210,6 @@ class ConnectionBorg(object):
         return self._get_connection().new_batch_list()
 
 
-
 class DynamoDBModel(object):
     """Abstract base class for all models that use DynamoDB as their storage
     backend.
@@ -200,14 +230,15 @@ class DynamoDBModel(object):
     from_dict (deserialization) and to_db_dict (serialization) methods.
 
     *Important implementation note regarding sets:* DynamoDB can't store empty
-    sets. Therefore, since we have schema information available to us, we're
-    storing empty sets as missing attributes in DynamoDB, and converting back
-    and forth based on the schema.
+    sets/strings. Therefore, since we have schema information available to us,
+    we're storing empty sets/strings as missing attributes in DynamoDB, and
+    converting back and forth based on the schema.
 
-    So if your schema looks like the following: {"id": unicode, "cheats": set},
-    then {"id": "e1m1", "cheats": set(["idkfa", "iddqd"])} will be stored
-    exactly as is, but {"id": "e1m2", "cheats": set()} will be stored as simply
-    {"id": "e1m1"}
+    So if your schema looks like the following: {"id": unicode, "name" str,
+    "cheats": set}, then {"id": "e1m1", "name": "Hangar",
+    "cheats": set(["idkfa", "iddqd"])} will be stored exactly as is, but
+    {"id": "e1m2", "name": "", "cheats": set()} will be stored as simply
+    {"id": "e1m2"}
 
 
     TODO Add checks for common error cases:
@@ -234,8 +265,12 @@ class DynamoDBModel(object):
         """
         instance = cls()
         for (name, type_) in cls.__schema__.iteritems():
-            # Fill in missing attributes with default values.
-            setattr(instance, name, d.get(name, type_()))
+            try:
+                value = type_(d[name])
+            except KeyError:
+                # Fill in missing attributes with default values.
+                value = type_()
+            setattr(instance, name, value)
         return instance
 
     @classmethod
@@ -330,10 +365,19 @@ class DynamoDBModel(object):
     def to_db_dict(self):
         """Return a dict representation of the object according to the class's
         schema, suitable for direct storage in DynamoDB.
-
-        This means the values must all be numbers, strings or sets thereof.
         """
-        return {name: getattr(self, name) for name in self.__schema__}
+        out = {}
+        for (name, type_) in self.__schema__.iteritems():
+            value = getattr(self, name)
+            # Yes, that part is horrible. DynamoDB can't store empty
+            # sets/strings, so we're representing them as missing
+            # attributes on the DB side.
+            if value or value == 0:
+                out[name] = value
+            elif type_ in [json_list, json_dict]:
+                # json serialization hooks for json_* data types.
+                out[name] = json.dumps(value)
+        return out
 
     def to_json_dict(self):
         """Return a dict representation of the object, suitable for JSON
@@ -387,12 +431,7 @@ class DynamoDBModel(object):
         the operation fails with OverwriteError).
         """
         table = ConnectionBorg().get_table(self.__table__)
-        # Yes, that part is horrible. DynamoDB can't store empty sets,
-        # so we're representing them as missing attributes on the DB side.
-        item_data = {
-            key: value for (key, value) in self.to_db_dict().iteritems() if (
-                value or not isinstance(value, (set, frozenset)))
-        }
+        item_data = self.to_db_dict()
         item = Item(table, attrs=item_data)
 
         if (self.__schema__[self.__hash_key__] == autoincrement_int and
